/*
 *  Copyright 2009 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.runtime.test;

import org.ametys.runtime.test.groups.jdbc.AllGroupsTestSuite;
import org.ametys.runtime.test.rights.profile.AllProfileBasedRightsManagerTestSuite;
import org.ametys.runtime.test.rights.profile.hierarchical.AllHierarchicalProfileBasedRightsManagerTestSuite;
import org.ametys.runtime.test.userpref.AllUserPreferencesTestSuite;
import org.ametys.runtime.test.users.jdbc.AllUsersTestSuite;
import org.ametys.runtime.test.users.jdbc.credentialsaware.AllCredentialsAwareJdbcUsersTestSuite;
import org.ametys.runtime.test.users.jdbc.modifiable.AllModifiableUsersTestSuite;
import org.ametys.runtime.test.users.jdbc.modifiablecredentialsaware.AllModifiableCredentialsAwareJdbcUsersTestSuite;

import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * Test suite grouping all JDBC-related runtime tests.
 */
public final class JdbcTestSuite
{
    private JdbcTestSuite()
    {
        // empty constructor
    }
    
    /**
     * Test suite
     * @return the Test suite
     */
    public static Test suite()
    {
        TestSuite suite = new TestSuite("All tests related to JDBC-based components");
        
        //$JUnit-BEGIN$
        
        suite.addTest(AllUsersTestSuite.suite());
        suite.addTest(AllModifiableUsersTestSuite.suite());
        suite.addTest(AllCredentialsAwareJdbcUsersTestSuite.suite());
        suite.addTest(AllModifiableCredentialsAwareJdbcUsersTestSuite.suite());
        
        suite.addTest(AllGroupsTestSuite.suite());
        
        suite.addTest(AllProfileBasedRightsManagerTestSuite.suite());
        suite.addTest(AllHierarchicalProfileBasedRightsManagerTestSuite.suite());
        
        suite.addTest(AllUserPreferencesTestSuite.suite());
        
        //$JUnit-END$
        return suite;
    }
}
